@@ -33,7 +33,9 @@ as a dictionary key.
     def __hash__(self):
         "Return hash(self)."
         if getattr(self, '_hash', None) is None:
-            self._hash = hash((len(self), self.tobytes()))
+            # ensure hash is independent of endianness
+            a = bitarray(self, 'big') if self.endian() == 'little' else self
+            self._hash = hash((len(a), a.tobytes()))
         return self._hash
 
     def __delitem__(self, *args, **kwargs):
@@ -44,19 +46,20 @@ as a dictionary key.
     frombytes = fromfile = insert = invert = pack = pop = __delitem__
     remove = reverse = setall = sort = __setitem__ = __delitem__
     __iadd__ = __iand__ = __imul__ = __ior__ = __ixor__ = __delitem__
+    __ilshift__ = __irshift__ = __delitem__
 
 
-def bits2bytes(_n):
+def bits2bytes(__n):
     """bits2bytes(n, /) -> int
 
 Return the number of bytes necessary to store n bits.
 """
     import sys
-    if not isinstance(_n, (int, long) if sys.version_info[0] == 2 else int):
+    if not isinstance(__n, (int, long) if sys.version_info[0] == 2 else int):
         raise TypeError("integer expected")
-    if _n < 0:
+    if __n < 0:
         raise ValueError("non-negative integer expected")
-    return 0 if _n == 0 else ((_n - 1) // 8 + 1)
+    return (__n + 7) // 8
 
 
 def test(verbosity=1, repeat=1):
