@@ -9,6 +9,7 @@
 
 #define PY_SSIZE_T_CLEAN
 #include "Python.h"
+#include "pythoncapi_compat.h"
 #include "bitarray.h"
 
 #define IS_LE(a)  ((a)->endian == ENDIAN_LITTLE)
@@ -36,6 +37,19 @@ ensure_bitarray(PyObject *obj)
     return 0;
 }
 
+/* ensure object is a bitarray of given length */
+static int
+ensure_ba_of_length(PyObject *a, const Py_ssize_t n)
+{
+    if (ensure_bitarray(a) < 0)
+        return -1;
+    if (((bitarrayobject *) a)->nbits != n) {
+        PyErr_SetString(PyExc_ValueError, "size mismatch");
+        return -1;
+    }
+    return 0;
+}
+
 /* ------------------------------- count_n ----------------------------- */
 
 /* return the smallest index i for which a.count(1, 0, i) == n, or when
@@ -334,7 +348,7 @@ serialize(PyObject *module, PyObject *a)
         return NULL;
 
     nbytes = Py_SIZE(a);
-    data = (char *) PyMem_Malloc(nbytes + 1);
+    data = (char *) PyMem_Malloc((size_t) nbytes + 1);
     if (data == NULL)
         return PyErr_NoMemory();
 
@@ -427,15 +441,10 @@ hex2ba(PyObject *module, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "Os#", &a, &str, &strsize))
         return NULL;
-    if (ensure_bitarray(a) < 0)
+    if (ensure_ba_of_length(a, 4 * strsize) < 0)
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
-    if (aa->nbits != 4 * strsize) {
-        PyErr_SetString(PyExc_ValueError, "size mismatch");
-        return NULL;
-    }
-
     le = IS_LE(aa);
     be = IS_BE(aa);
     assert(le + be == 1 && str[strsize] == 0);
@@ -504,11 +513,11 @@ digit_to_int(char c, int n)
 static int
 base_to_length(int n)
 {
-    int k;
+    int m;
 
-    for (k = 1; k < 7; k++) {
-        if (n == (1 << k))
-            return k;
+    for (m = 1; m < 7; m++) {
+        if (n == (1 << m))
+            return m;
     }
     PyErr_SetString(PyExc_ValueError, "base must be 2, 4, 8, 16, 32 or 64");
     return -1;
@@ -523,12 +532,12 @@ ba2base(PyObject *module, PyObject *args)
     char *str;
     int n, m, x, k, le;
 
-    if (!PyArg_ParseTuple(args, "iO:ba2ascii", &n, &a))
-        return NULL;
-    if (ensure_bitarray(a) < 0)
+    if (!PyArg_ParseTuple(args, "iO:ba2base", &n, &a))
         return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
+    if (ensure_bitarray(a) < 0)
+        return NULL;
 
     switch (n) {
     case 32: alphabet = base32_alphabet; break;
@@ -586,16 +595,12 @@ base2ba(PyObject *module, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "iOs#", &n, &a, &str, &strsize))
         return NULL;
-    if (ensure_bitarray(a) < 0)
-        return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
+    if (ensure_ba_of_length(a, m * strsize) < 0)
+        return NULL;
 
 #define aa  ((bitarrayobject *) a)
-    if (aa->nbits != m * strsize) {
-        PyErr_SetString(PyExc_ValueError, "size mismatch");
-        return NULL;
-    }
     memset(aa->ob_item, 0x00, (size_t) Py_SIZE(a));
 
     le = IS_LE(aa);
@@ -612,6 +617,152 @@ base2ba(PyObject *module, PyObject *args)
     Py_RETURN_NONE;
 }
 
+
+#define PADBITS  3        /* number of padding bits, used in vl_*() */
+
+/* Consume iterator while decoding bytes into bitarray.
+   As we don't have access to bitarrays resize() C function, we give this
+   function a bitarray (large enough in most cases).  We manipulate .nbits
+   and .ob_size (using Py_SET_SIZE) directly without having to call resize().
+   Whenever we need a larger bitarray, we call .frombytes() with a multiple
+   of 7 dummy bytes, such that the added bytes are aligned for the next time
+   we call .frombytes() (to avoid expensive bit shifts).
+   We drop the over-allocated bitarray on the Python side after this function
+   is called.
+*/
+static PyObject *
+vl_decode(PyObject *module, PyObject *args)
+{
+    const Py_ssize_t ibits = 256;    /* initial number of bits in a */
+    PyObject *iter, *item, *res, *a;
+    Py_ssize_t padding = 0;    /* uninitialized warning for some compiles */
+    Py_ssize_t k;
+    Py_ssize_t i = 0;          /* bit counter */
+    unsigned char b = 0x80;    /* empty stream will raise StopIteration */
+
+    assert(ibits == 37 * 7 - PADBITS);   /* bits in a 37 byte stream */
+    assert(ibits % 8 == 0);     /* bytes being added are aligned */
+
+    if (!PyArg_ParseTuple(args, "OO", &iter, &a))
+        return NULL;
+    if (!PyIter_Check(iter))
+        return PyErr_Format(PyExc_TypeError, "iterator or bytes expected, "
+                            "got '%s'", Py_TYPE(iter)->tp_name);
+    if (ensure_ba_of_length(a, ibits) < 0)
+        return NULL;
+
+    while ((item = PyIter_Next(iter))) {
+#ifdef IS_PY3K
+        if (PyLong_Check(item))
+            b = (unsigned char) PyLong_AsLong(item);
+#else
+        if (PyBytes_Check(item))
+            b = (unsigned char) *PyBytes_AS_STRING(item);
+#endif
+        else {
+            PyErr_Format(PyExc_TypeError, "int (byte) iterator expected, "
+                         "got '%s' element", Py_TYPE(item)->tp_name);
+            Py_DECREF(item);
+            return NULL;
+        }
+        Py_DECREF(item);
+
+#define aa  ((bitarrayobject *) a)
+        if (i + 6 >= BITS(Py_SIZE(aa))) {
+            /* grow memory - see above */
+            assert(i % 8 == 0);  /* ensure added dummy bytes are aligned */
+            aa->nbits = i;
+            Py_SET_SIZE(aa, BYTES(i));
+            /* 63 is a multiple 7, such bytes are aligned for next call */
+            res = PyObject_CallMethod(a, "frombytes", BYTES_SIZE_FMT,
+                                      base64_alphabet, (Py_ssize_t) 63);
+            if (res == NULL)
+                return NULL;
+            Py_DECREF(res);  /* drop extend result */
+        }
+        assert(i + 6 < BITS(Py_SIZE(aa)));
+
+        if (i == 0) {
+            padding = (b & 0x70) >> 4;
+            if (padding >= 7 || ((b & 0x80) == 0 && padding > 4))
+                return PyErr_Format(PyExc_ValueError,
+                                    "invalid header byte: 0x%02x", b);
+            for (k = 0; k < 4; k++)
+                setbit(aa, i++, (0x08 >> k) & b);
+        }
+        else {
+            for (k = 0; k < 7; k++)
+                setbit(aa, i++, (0x40 >> k) & b);
+        }
+        if ((b & 0x80) == 0)
+            break;
+    }
+    assert(i == 0 || (i + PADBITS) % 7 == 0);
+    /* set final length of bitarray */
+    aa->nbits = i - padding;
+    Py_SET_SIZE(aa, BYTES(aa->nbits));
+#undef aa
+
+    if (PyErr_Occurred())       /* from PyIter_Next() */
+        return NULL;
+
+    if (b & 0x80) {
+        k = (i + PADBITS) / 7;
+        return PyErr_Format(PyExc_StopIteration,
+                            "no terminating byte found, bytes read: %zd", k);
+    }
+    Py_RETURN_NONE;
+}
+
+static PyObject *
+vl_encode(PyObject *module, PyObject *a)
+{
+    PyObject *result;
+    Py_ssize_t padding, n, m, i, k;
+    Py_ssize_t j = 0;           /* byte conter */
+    char *data;
+
+    if (ensure_bitarray(a) < 0)
+        return NULL;
+
+#define aa  ((bitarrayobject *) a)
+    n = (aa->nbits + PADBITS + 6) / 7;  /* number of resulting bytes */
+    m = 7 * n - PADBITS;      /* number of bits resulting bytes can hold */
+    padding = m - aa->nbits;  /* number of pad bits */
+    assert(0 <= padding && padding < 7);
+
+    if ((data = (char *) PyMem_Malloc((size_t) n)) == NULL)
+        return PyErr_NoMemory();
+
+    data[0] = aa->nbits > 4 ? 0x80 : 0x00;  /* leading bit */
+    data[0] |= padding << 4;                /* encode number of pad bits */
+    for (i = 0; i < 4 && i < aa->nbits; i++)
+        if (GETBIT(aa, i))
+            data[0] |= 0x08 >> i;
+
+    for (i = 4; i < aa->nbits; i++) {
+        k = (i - 4) % 7;
+        if (k == 0)
+            data[++j] = i + 7 < m ? 0x80 : 0x00;  /* leading bit */
+
+        if (GETBIT(aa, i))
+            data[j] |= 0x40 >> k;
+    }
+#undef aa
+    assert(j + 1 == n);
+
+    result = PyBytes_FromStringAndSize(data, n);
+    PyMem_Free((void *) data);
+    return result;
+}
+
+PyDoc_STRVAR(vl_encode_doc,
+"vl_encode(bitarray, /) -> bytes\n\
+\n\
+Return variable length binary representation of bitarray.\n\
+This representation is useful for efficiently storing small bitarray\n\
+in a binary stream.  Use `vl_decode()` for decoding.");
+
 /* --------------------------------------------------------------------- */
 
 /* Set bitarray_type_obj (bato).  This function must be called before any
@@ -636,6 +787,8 @@ static PyMethodDef module_functions[] = {
     {"_hex2ba",   (PyCFunction) hex2ba,    METH_VARARGS, 0},
     {"ba2base",   (PyCFunction) ba2base,   METH_VARARGS, ba2base_doc},
     {"_base2ba",  (PyCFunction) base2ba,   METH_VARARGS, 0},
+    {"vl_encode", (PyCFunction) vl_encode, METH_O,       vl_encode_doc},
+    {"_vl_decode",(PyCFunction) vl_decode, METH_VARARGS, 0},
     {"_set_bato", (PyCFunction) set_bato,  METH_O,       0},
     {NULL,        NULL}  /* sentinel */
 };
