@@ -4,6 +4,7 @@ Tests for bitarray.util module
 from __future__ import absolute_import
 
 import os
+import re
 import sys
 import base64
 import binascii
@@ -22,7 +23,7 @@ from bitarray.util import (
     zeros, urandom, pprint, make_endian, rindex, strip, count_n,
     parity, count_and, count_or, count_xor, subset,
     serialize, deserialize, ba2hex, hex2ba, ba2base, base2ba,
-    ba2int, int2ba, huffman_code
+    ba2int, int2ba, vl_encode, vl_decode, huffman_code
 )
 
 if sys.version_info[0] == 3:
@@ -879,6 +880,146 @@ tests.append(TestsBase)
 
 # ---------------------------------------------------------------------------
 
+class VLFTests(unittest.TestCase, Util):
+
+    def test_explicit(self):
+        for s, bits in [
+                (b'\x40', ''),
+                (b'\x30', '0'),
+                (b'\x38', '1'),
+                (b'\x00', '0000'),
+                (b'\x01', '0001'),
+                (b'\xe0\x40', '0000 1'),
+                (b'\x90\x02', '0000 000001'),
+                (b'\xb5\xa7\x18', '0101 0100111 0011'),
+        ]:
+            a = bitarray(bits)
+            self.assertEqual(vl_encode(a), s)
+            self.assertEqual(vl_decode(s), a)
+
+    def test_encode(self):
+        for endian in 'big', 'little':
+            s = vl_encode(bitarray('001101', endian))
+            self.assertIsInstance(s, bytes)
+            self.assertEqual(s, b'\xd3\x20')
+
+    def test_decode_args(self):
+        if sys.version_info[0] == 3:
+            self.assertRaises(TypeError, vl_decode, 'foo')
+            self.assertRaises(TypeError, vl_decode, iter([b'\x40']))
+
+        self.assertRaises(TypeError, vl_decode, b'\x40', 'big', 3)
+        self.assertRaises(ValueError, vl_decode, b'\x40', 'foo')
+        for item in None, 2.34, Ellipsis:
+            self.assertRaises(TypeError, vl_decode, iter([item]))
+
+        lst = [b'\xd3\x20', iter(b'\xd3\x20')]
+        if sys.version_info[0] == 3:
+            lst.append(iter([0xd3, 0x20]))
+        for s in lst:
+            a = vl_decode(s, endian=self.random_endian())
+            self.assertIsInstance(a, bitarray)
+            self.assertEqual(a, bitarray('0011 01'))
+
+    def test_decode_endian(self):
+        for endian in 'little', 'big', None:
+            a = vl_decode(b'\xd3\x20', endian)
+            self.assertEqual(a, bitarray('0011 01'))
+            self.assertEqual(a.endian(),
+                             endian if endian else get_default_endian())
+
+    def test_decode_trailing(self):
+        for s, bits in [(b'\x40ABC', ''),
+                        (b'\xe0\x40A', '00001')]:
+            stream = iter(s)
+            self.assertEqual(vl_decode(stream), bitarray(bits))
+            self.assertEqual(next(stream),
+                             b'A' if sys.version_info[0] == 2 else 65)
+
+    def test_decode_ambiguity(self):
+        for s in b'\x40', b'\x4f', b'\x45':
+            self.assertEqual(vl_decode(iter(s)), bitarray())
+        for s in b'\x1e', b'\x1f':
+            self.assertEqual(vl_decode(iter(s)), bitarray('111'))
+
+    def test_decode_stream(self):
+        stream = iter(b'\x40\x30\x38\x40\x2c\xe0\x40\xd3\x20')
+        for bits in '', '0', '1', '', '11', '0000 1', '0011 01':
+            self.assertEqual(vl_decode(stream), bitarray(bits))
+
+        arrays = [urandom(randint(0, 30)) for _ in range(1000)]
+        stream = iter(b''.join(vl_encode(a) for a in arrays))
+        for a in arrays:
+            self.assertEqual(vl_decode(stream), a)
+
+    def test_decode_errors(self):
+        # decode empty bits
+        self.assertRaises(StopIteration, vl_decode, b'')
+        # invalid number of padding bits
+        for s in b'\x50', b'\x60', b'\x70':
+            self.assertRaises(ValueError, vl_decode, s)
+        self.assertRaises(ValueError, vl_decode, b'\xf0')
+        # high bit set, but no terminating byte
+        for s in b'\x80', b'\x80\x80':
+            self.assertRaises(StopIteration, vl_decode, s)
+
+    def test_decode_error_message(self):
+        pat = re.compile(r'[\w\s,]+:\s+(\d+)')
+        for n in range(120):
+            a = None
+            s = bytes(bytearray([randint(0x80, 0xef) for _ in range(n)]))
+            try:
+                a = vl_decode(s)
+            except StopIteration as e:
+                m = pat.match(str(e))
+                self.assertEqual(m.group(1), str(n))
+            self.assertTrue(a is None)
+
+    def test_decode_invalid_stream(self):
+        if sys.version_info[0] == 2:
+            return
+        N = 100
+        s = iter(N * (3 * [0x80] + ['XX']) + ['end.'])
+        for _ in range(N):
+            a = None
+            try:
+                a = vl_decode(s)
+            except TypeError:
+                pass
+            self.assertTrue(a is None)
+        self.assertEqual(next(s), 'end.')
+
+    def test_explicit_zeros(self):
+        for n in range(100):
+            a = zeros(4 + n * 7)
+            s = n * b'\x80' + b'\x00'
+            self.assertEqual(vl_encode(a), s)
+            self.assertEqual(vl_decode(s), a)
+
+    def round_trip(self, a):
+        s = vl_encode(a)
+        b = vl_decode(s)
+        self.check_obj(b)
+        self.assertEqual(a, b)
+        padding = 3
+        self.assertEqual(len(s), (len(a) + padding + 6) // 7)
+
+    def test_range(self):
+        for n in range(500):
+            self.round_trip(bitarray(n))
+
+    def test_large(self):
+        a = urandom(randint(50000, 100000))
+        self.round_trip(a)
+
+    def test_random(self):
+        for a in self.randombitarrays():
+            self.round_trip(a)
+
+tests.append(VLFTests)
+
+# ---------------------------------------------------------------------------
+
 class TestsIntegerization(unittest.TestCase, Util):
 
     def test_ba2int(self):
